@@ -38,7 +38,6 @@
 
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <gnome-keyring-memory.h>
 
 #define NM_VPN_API_SUBJECT_TO_CHANGE
 
@@ -454,6 +453,7 @@ init_plugin_ui (VpncPluginUiWidget *self,
                 GError **error)
 {
 	VpncPluginUiWidgetPrivate *priv = VPNC_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
+	NMSettingConnection *s_con = NULL;
 	NMSettingVPN *s_vpn = NULL;
 	GtkWidget *widget;
 	GtkListStore *store;
@@ -467,8 +467,10 @@ init_plugin_ui (VpncPluginUiWidget *self,
 	gboolean enabled = FALSE;
 	GtkFileFilter *filter;
 
-	if (connection)
+	if (connection) {
+		s_con = nm_connection_get_setting_connection (connection);
 		s_vpn = nm_connection_get_setting_vpn (connection);
+	}
 
 	priv->group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
@@ -608,6 +610,16 @@ init_plugin_ui (VpncPluginUiWidget *self,
 	}
 	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (stuff_changed_cb), self);
 
+	/* Interface name */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "interface_name_entry"));
+	g_return_val_if_fail (widget != NULL, FALSE);
+	if (s_con) {
+		value = nm_setting_connection_get_interface_name (s_con);
+		if (value && strlen (value))
+			gtk_entry_set_text (GTK_ENTRY (widget), value);
+	}
+	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (stuff_changed_cb), self);
+
 	active = -1;
 	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
 	if (s_vpn)
@@ -893,6 +905,7 @@ update_connection (NMVpnPluginUiWidgetInterface *iface,
 {
 	VpncPluginUiWidget *self = VPNC_PLUGIN_UI_WIDGET (iface);
 	VpncPluginUiWidgetPrivate *priv = VPNC_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
+	NMSettingConnection *s_con;
 	NMSettingVPN *s_vpn;
 	GtkWidget *widget;
 	char *str;
@@ -903,9 +916,17 @@ update_connection (NMVpnPluginUiWidgetInterface *iface,
 	if (!check_validity (self, error))
 		return FALSE;
 
+	s_con = nm_connection_get_setting_connection (connection);
+
 	s_vpn = NM_SETTING_VPN (nm_setting_vpn_new ());
 	g_object_set (s_vpn, NM_SETTING_VPN_SERVICE_TYPE, NM_DBUS_SERVICE_VPNC, NULL);
 
+	/* Interface name */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "interface_name_entry"));
+	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
+	if (str && strlen (str))
+		g_object_set (G_OBJECT (s_con), NM_SETTING_CONNECTION_INTERFACE_NAME, str, NULL);
+
 	/* Gateway */
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "gateway_entry"));
 	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
@@ -1297,6 +1318,24 @@ decrypt_cisco_key (const char* enc_key)
 	return key;
 }
 
+typedef enum {
+	NM_VPNC_IMPORT_EXPORT_ERROR_UNKNOWN = 0,
+	NM_VPNC_IMPORT_EXPORT_ERROR_NOT_VPNC,
+	NM_VPNC_IMPORT_EXPORT_ERROR_BAD_DATA,
+} NMVpncImportError;
+
+#define NM_VPNC_IMPORT_EXPORT_ERROR nm_vpnc_import_export_error_quark ()
+
+static GQuark
+nm_vpnc_import_export_error_quark (void)
+{
+	static GQuark quark = 0;
+
+	if (G_UNLIKELY (quark == 0))
+		quark = g_quark_from_static_string ("nm-vpnc-import-export-error-quark");
+	return quark;
+}
+
 static NMConnection *
 import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 {
@@ -1312,11 +1351,8 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 	gboolean found;
 
 	keyfile = g_key_file_new ();
-	if (!g_key_file_load_from_file (keyfile, path, 0, error)) {
-		g_set_error (error, 0, 0, "does not look like a %s VPN connection (parse failed)",
-		             VPNC_PLUGIN_NAME);
+	if (!g_key_file_load_from_file (keyfile, path, 0, error))
 		goto error;
-	}
 
 	connection = nm_connection_new ();
 	s_con = NM_SETTING_CONNECTION (nm_setting_connection_new ());
@@ -1329,13 +1365,23 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 	s_ip4 = NM_SETTING_IP4_CONFIG (nm_setting_ip4_config_new ());
 	nm_connection_add_setting (connection, NM_SETTING (s_ip4));
 
+	/* Interface Name */
+	buf = key_file_get_string_helper (keyfile, "main", "InterfaceName", NULL);
+	if (buf) {
+		g_object_set (G_OBJECT (s_con), NM_SETTING_CONNECTION_INTERFACE_NAME, buf, NULL);
+		g_free (buf);
+	}
+
 	/* Gateway */
 	buf = key_file_get_string_helper (keyfile, "main", "Host", NULL);
 	if (buf) {
 		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_GATEWAY, buf);
 		g_free (buf);
 	} else {
-		g_set_error (error, 0, 0, "does not look like a %s VPN connection (no Host)",
+		g_set_error (error,
+		             NM_VPNC_IMPORT_EXPORT_ERROR,
+		             NM_VPNC_IMPORT_EXPORT_ERROR_NOT_VPNC,
+		             "does not look like a %s VPN connection (no Host)",
 		             VPNC_PLUGIN_NAME);
 		goto error;
 	}
@@ -1346,7 +1392,10 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_ID, buf);
 		g_free (buf);
 	} else {
-		g_set_error (error, 0, 0, "does not look like a %s VPN connection (no GroupName)",
+		g_set_error (error,
+		             NM_VPNC_IMPORT_EXPORT_ERROR,
+		             NM_VPNC_IMPORT_EXPORT_ERROR_BAD_DATA,
+		             "does not look like a %s VPN connection (no GroupName)",
 		             VPNC_PLUGIN_NAME);
 		goto error;
 	}
@@ -1562,6 +1611,7 @@ export (NMVpnPluginUiInterface *iface,
 	FILE *f;
 	const char *value;
 	const char *gateway = NULL;
+	GString *interface_name = NULL;
 	gboolean enablenat = TRUE;
 	gboolean singledes = FALSE;
 	const char *groupname = NULL;
@@ -1587,15 +1637,26 @@ export (NMVpnPluginUiInterface *iface,
 
 	f = fopen (path, "w");
 	if (!f) {
-		g_set_error (error, 0, 0, "could not open file for writing");
+		g_set_error_literal (error,
+		                     NM_VPNC_IMPORT_EXPORT_ERROR,
+		                     NM_VPNC_IMPORT_EXPORT_ERROR_UNKNOWN,
+		                     "could not open file for writing");
 		return FALSE;
 	}
 
+	interface_name = g_string_new("");
+	value = nm_setting_connection_get_interface_name (s_con);
+	if (value && strlen (value))
+		g_string_printf (interface_name, "InterfaceName=%s\n", value);
+
 	value = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_GATEWAY);
 	if (value && strlen (value))
 		gateway = value;
 	else {
-		g_set_error (error, 0, 0, "connection was incomplete (missing gateway)");
+		g_set_error_literal (error,
+		                     NM_VPNC_IMPORT_EXPORT_ERROR,
+		                     NM_VPNC_IMPORT_EXPORT_ERROR_BAD_DATA,
+		                     "connection was incomplete (missing gateway)");
 		goto done;
 	}
 
@@ -1603,7 +1664,10 @@ export (NMVpnPluginUiInterface *iface,
 	if (value && strlen (value))
 		groupname = value;
 	else {
-		g_set_error (error, 0, 0, "connection was incomplete (missing group)");
+		g_set_error_literal (error,
+		                     NM_VPNC_IMPORT_EXPORT_ERROR,
+		                     NM_VPNC_IMPORT_EXPORT_ERROR_BAD_DATA,
+		                     "connection was incomplete (missing group)");
 		goto done;
 	}
 
@@ -1700,6 +1764,7 @@ export (NMVpnPluginUiInterface *iface,
 	fprintf (f, 
 		 "[main]\n"
 		 "Description=%s\n"
+		 "%s"
 		 "Host=%s\n"
 		 "AuthType=1\n"
 		 "GroupName=%s\n"
@@ -1741,6 +1806,7 @@ export (NMVpnPluginUiInterface *iface,
 		 "X-NM-SaveGroupPassword=%s\n"
 		 "%s\n",
 		 /* Description */   nm_setting_connection_get_id (s_con),
+		 /* InterfaceName */ (interface_name->len) ? interface_name->str : "",
 		 /* Host */          gateway,
 		 /* GroupName */     groupname,
 		 /* GroupPassword */ group_pw ? group_pw : "",
@@ -1760,6 +1826,8 @@ export (NMVpnPluginUiInterface *iface,
 	success = TRUE;
 
 done:
+	if (interface_name)
+		g_string_free (interface_name, TRUE);
 	if (routes)
 		g_string_free (routes, TRUE);
 	if (uselegacyikeport)
@@ -1860,6 +1928,9 @@ nm_vpn_plugin_ui_factory (GError **error)
 	if (error)
 		g_return_val_if_fail (*error == NULL, NULL);
 
+	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
 	return NM_VPN_PLUGIN_UI_INTERFACE (g_object_new (VPNC_TYPE_PLUGIN_UI, NULL));
 }
 
